/*
 * This example sketch shows how you can manage the nRF5 pin mapping as part of your code.
 * You can use the method for any nRF51822 or nRF52832 board or module.
 * 
 * Most components, like UART, SPI, Wire Bus, of the nRF5 series chips don't
 * have a fixed pin mapping. There are some pins with restrictions like analog
 * inputs, NFC or pins near the radio module. Please refer the latest
 * documentation about pin restrictions at http://infocenter.nordicsemi.com 
 * 
 * To use the custom pin mapping you have to do following steps:
 * 
 * 1. Install "arduino-nrf5" like described at
 *    https://github.com/sandeepmistry/arduino-nRF5/
 * 2. Install the "My Sensors nRF5 Boards" with the board manager like
 *    explained at https://github.com/mysensors/ArduinoBoards
 * 3. Copy the files "MyBoardNRF5.cpp" and "MyBoardNRF5.h" from
 *    "MyBoardNRF5" example into your sketch.
 * 4. Modify pin mappings in "MyBoardNRF5.cpp" and "MyBoardNRF5.h" to fit
 *    your requirements.
 * 5. Select "MyBoardNRF5 nrf52832" or "MyBoardNRF5 nrf52822" as your board.
 *    Choose the correct parameters and programmer in the Tools menu.
 */

/* Module DIN 6xPilotes Gite 1
 *  LED verte Power
 *  LED Rouge état Pilote (6x)
 *    P0.20 -> LED D4 Pilote 1
 *    P0.25 -> LED D8 Pilote 2
 *    P0.19 -> LED D3 Pilote 3
 *    P0.23 -> LED D6 Pilote 4
 *    P0.18 -> LED D2 Pilote 5
 *    P0.24 -> LED D7 Pilote 6
 *  LED Bleu Communication Radio
 *    P0.22 -> LED D5 Comunication
 */

//#define MY_DEBUG
//#define MY_SPECIAL_DEBUG
//#define MY_DEBUG_VERBOSE
//#define MY_DEBUG_VERBOSE_NRF5_ESB
// #define MY_TRANSPORT_SANITY_CHECK


#define MY_RADIO_NRF5_ESB
// Define my own network ID (Not default one)
#define MY_NRF5_ESB_BASE_RADIO_ID 0x00,0xFC,0xE1,0x22,0x22
#define MY_NODE_ID 121
//#define MY_PASSIVE_NODE
//#define MY_PARENT_NODE_ID 0

// Inverses the behavior of leds
#define MY_WITH_LEDS_BLINKING_INVERSE

// Flash leds on rx/tx/err
// Uncomment to override default HW configurations
//#define MY_DEFAULT_ERR_LED_PIN 22  // Error led pin
#define MY_DEFAULT_RX_LED_PIN  22  // Receive led pin
#define MY_DEFAULT_TX_LED_PIN  22  // Transmit led pin

#define INTER_MESSAGES_WAIT 50

#include <MySensors.h>
#include <Wire.h>
#include "Adafruit_MCP23017.h"

#define LED_ON 0
#define LED_OFF 1

#define LED_PILOTE_1 20 
#define LED_PILOTE_2 25 
#define LED_PILOTE_3 19
#define LED_PILOTE_4 23
#define LED_PILOTE_5 18
#define LED_PILOTE_6 24

#define NB_LED_PILOTE 6

const int LEDS_PILOTE[6] = {
  LED_PILOTE_1,
  LED_PILOTE_2,
  LED_PILOTE_3,
  LED_PILOTE_4,
  LED_PILOTE_5,
  LED_PILOTE_6
};

typedef struct {
   char mode_code;
   int code_A;
   int code_B;
} modeFilPilote;

const modeFilPilote modesFilPilote[4] = {
  {'O', 0, 1}, // Mode Off
  {'G', 1, 0}, // Mode hors gel
  {'E', 0, 0}, // Mode éco
  {'C', 1, 1}  // Mode confort
};

typedef struct {
   int childID;
   char *name;
   char lastOrder;
   int pin_A;
   int pin_B;
   bool newState;
} FilPilote;

FilPilote ListFilPilote[6] = {
  {0, "Pilote1", 'O', 0, 13, true},
  {1, "Pilote2", 'O', 1, 12, true},
  {2, "Pilote3", 'O', 2, 11, true},
  {3, "Pilote4", 'O', 3, 10, true},
  {4, "Pilote5", 'O', 4, 9, true},
  {5, "Pilote6", 'O', 5, 8, true}
};

#define NB_PILOTE 6

int LedTimer = 0;
int BatteryTimer = 0;

MyMessage msg_S_HEATER(14,V_STATUS); 

Adafruit_MCP23017 mcp;

//-------------------------------------
void nRF_Init() {
  NRF_POWER->DCDCEN = 0; // Not reducing the radio current   
  NRF_PWM0  ->ENABLE = 0;
  NRF_PWM1  ->ENABLE = 0;
  NRF_PWM2  ->ENABLE = 0;
  NRF_TWIM1 ->ENABLE = 0;
  NRF_TWIS1 ->ENABLE = 0;
  NRF_RADIO->TXPOWER = 4;
}

void disableNfc() {
  NRF_NFCT->TASKS_DISABLE = 1;
  NRF_NVMC->CONFIG = 1;
  NRF_UICR->NFCPINS = 0;
  NRF_NVMC->CONFIG = 0;
}

void turnOffAdc() {
  if (NRF_SAADC->ENABLE) {
    NRF_SAADC->TASKS_STOP = 1;
    while (NRF_SAADC->EVENTS_STOPPED) {}
    NRF_SAADC->ENABLE = 0;
    while (NRF_SAADC->ENABLE) {}
  }
}

//void sleepI2C() {
//  NRF_TWI1->ENABLE=TWI_ENABLE_ENABLE_Disabled << TWI_ENABLE_ENABLE_Pos;  // I2C Begin is then required on resume (sensor.begin();)
//  *(volatile uint32_t *)0x40004FFC = 0;
//  *(volatile uint32_t *)0x40004FFC;
//  *(volatile uint32_t *)0x40004FFC = 1;
//}
//---------------------------------------

void preHwInit() 
{ 
  // Configure Leds ports 
  pinMode(22, OUTPUT);  
  
  for (int i = 0; i < NB_LED_PILOTE; i++) {
    pinMode(LEDS_PILOTE[i], OUTPUT); 
    digitalWrite(LEDS_PILOTE[i], LED_OFF); 
  }  

  // Test Leds
  for(int i=0; i < NB_LED_PILOTE +1; i++)
  {
    if (i-1>=0)digitalWrite(LEDS_PILOTE[i-1], LED_OFF);
    if (i < NB_LED_PILOTE)digitalWrite(LEDS_PILOTE[i], LED_ON);
    wait(250);
  }  
}

void before()
{    
  nRF_Init();
  disableNfc();
  turnOffAdc();       
  wait(200);
}

void setup() 
{  
  mcp.begin();      // use default address 0  

  // Load Pilot saved states
  for (int i = 0; i < NB_PILOTE; i++) {
    ListFilPilote[i].lastOrder = loadState(ListFilPilote[i].childID);    
    #ifdef MY_DEBUG
          Serial.print("--Pilote: ");
          Serial.print(ListFilPilote[i].name);
          Serial.print("  --Restored Status: ");
          Serial.println(ListFilPilote[i].lastOrder);          
    #endif
  }     

  // Configure MCP ports
  for (int i = 0; i <= 15; i++) {
    mcp.pinMode(i, OUTPUT);
    mcp.digitalWrite(0, HIGH);
  }

  // Set MCP ports
  for(int p=0; p < NB_PILOTE; p++) 
  {
    for(int e=0; e < 4; e++)
    {
      if(ListFilPilote[p].lastOrder == modesFilPilote[e].mode_code)
      {
        // Set new state
        mcp.digitalWrite(ListFilPilote[p].pin_A, modesFilPilote[e].code_A); 
        mcp.digitalWrite(ListFilPilote[p].pin_B, modesFilPilote[e].code_B);        
        #ifdef MY_DEBUG
          Serial.print("--Pilote: ");
          Serial.print(ListFilPilote[p].name);
          Serial.print("  --Restored Status Pin A: ");
          Serial.print(modesFilPilote[e].code_A);          
          Serial.print("  --Restored Status Pin B: ");
          Serial.println(modesFilPilote[e].code_B);          
        #endif        
      }
    }
  }
}

void presentation()
{
  sendSketchInfo("DIN 6xPILOTES Gite 1", "0.8");
  wait(INTER_MESSAGES_WAIT);  

  // Present Pilote
  for (int i = 0; i < NB_PILOTE; i++) 
  {
    present(ListFilPilote[i].childID, S_HEATER, ListFilPilote[i].name);  
    wait(INTER_MESSAGES_WAIT); 
  }   
}

void loop() 
{
  /*
   * Off: Half upper                 Led: Off
   * Confort: Nothing                Led: On 
   * Eco: Half upper & lower (Full)  Led: Blinking 50/50
   * Frizing Protect: Half lower     Led: Blinking 10/90
   */
   
  for(int i=0; i < NB_LED_PILOTE; i++)
  {
    switch (ListFilPilote[i].lastOrder)
    {      
      case 'G': // horsgel
       if (LedTimer <9) digitalWrite(LEDS_PILOTE[i], LED_OFF); 
       else digitalWrite(LEDS_PILOTE[i], LED_ON);        
       break;
      case 'E': // Eco
       if (LedTimer < 5) digitalWrite(LEDS_PILOTE[i], LED_ON); 
       else digitalWrite(LEDS_PILOTE[i], LED_OFF); 
       break;
      case 'O': // off
       digitalWrite(LEDS_PILOTE[i], LED_OFF); 
       break;
      case 'C': // Confort
       digitalWrite(LEDS_PILOTE[i], LED_ON); 
       break;
    }       
  }
  LedTimer++;
  if (LedTimer > 9) LedTimer = 0;
  BatteryTimer++;
  if (BatteryTimer > 599)
  {
    BatteryTimer = 0;
    #ifdef MY_DEBUG
      Serial.println("Send Battery Level");      
    #endif
    sendBatteryLevel(100);
    //  sendHeartbeat();
  }
  
  wait(100);
}

void receive(const MyMessage &message)
{   
 if (message.type == V_HVAC_FLOW_STATE)
  {
    #ifdef MY_DEBUG
          Serial.print("--Incoming message: ");
          Serial.print(message.sensor);
          Serial.print("  Message: ");
          Serial.print(message.getString()[0]);
          Serial.println();
    #endif
    for(int p=0; p < NB_PILOTE; p++) 
    {
      if (message.sensor == ListFilPilote[p].childID)
      {
        #ifdef MY_DEBUG
          Serial.print("--Incoming change for: ");
          Serial.print(ListFilPilote[p].name);
          Serial.print("  --Command: ");
          Serial.print(message.getString()[0]);
          Serial.println();
        #endif

        for(int e=0; e < 4; e++)
        {
          if(message.getString()[0] == modesFilPilote[e].mode_code)
          {
            // Set new state
            mcp.digitalWrite(ListFilPilote[p].pin_A, modesFilPilote[e].code_A); 
            mcp.digitalWrite(ListFilPilote[p].pin_B, modesFilPilote[e].code_B);        
            
            // Save state
            ListFilPilote[p].lastOrder = modesFilPilote[e].mode_code;
            saveState(ListFilPilote[p].childID, modesFilPilote[e].mode_code); 

            // Send status update
            //send(msg_S_HEATER.setSensor(ListFilPilote[p].childID).set((const char *)(modesFilPilote[e].mode_code)));
            return;
          }
        }
        // Unknown Order
        #ifdef MY_DEBUG
          Serial.print("--Unknown Order: ");
          Serial.println(message.getString()[0]);
        #endif
      }      
    }
    // Wrong Child_ID
    #ifdef MY_DEBUG
        Serial.print("--Wrong ChildID: ");
        Serial.println(message.sensor);
    #endif
  }
}

  
  
/* 
void storeEeprom(int pos, int value) {
  // function for saving the values to the internal EEPROM
  // value = the value to be stored (as int)
  // pos = the first byte position to store the value in
  // only two bytes can be stored with this function (max 32.767)
  gw.saveState(pos, ((unsigned int)value >> 8 ));
  pos++;
  gw.saveState(pos, (value & 0xff));
}

int readEeprom(int pos) {
  // function for reading the values from the internal EEPROM
  // pos = the first byte position to read the value from 

  int hiByte;
  int loByte;

  hiByte = gw.loadState(pos) << 8;
  pos++;
  loByte = gw.loadState(pos);
  return (hiByte | loByte);
}

You can also use a data structure to define a dictionary:

typedef struct { 
  uint8_t lang;
  char* sound;
  char* value;
} langDictionary;
Then you can define an array of the structure with the values to be used:

const langDictionary myDictionaryArr[] {
    {0, "ENGLISH", "Settings"},
    {1, "SPANISH", "Ajustes"},
    {2, "FRENCH", "Paramètres"}
};
And finally you can use the array to search the properties:

void setup() {
  Serial.begin(115200);
  for(uint8_t i = 0; i < sizeof(myDictionaryArr)/sizeof(langDictionary); ++i) {
      Serial.println(myDictionaryArr[i].value); //Prints the values: "Settings", "Ajustes" and "Paramètres"
  }

*/
